@@ -18,7 +18,7 @@ from cryspy.E_data_classes.cl_1_crystal import Crystal
 from cryspy.E_data_classes.cl_2_diffrn import Diffrn
 
 from cryspy.F_functions_data.script_1_mem import maximize_entropy, \
-    refine_susceptibility, make_cycle
+    refine_susceptibility, make_cycle, calc_moments_in_unit_cell
 
 
 class MEM(GlobalN):
@@ -84,14 +84,20 @@ class MEM(GlobalN):
     def save_to_file_den(self, f_name: str = "file.den", label_atom=None,
                          f_background: str = "file_back.den"):
         """Save density to ".den" files."""
-        mem_parameters = self.mem_parameters
         crystal = self.crystals()[0]
         space_group = crystal.space_group
         cell = crystal.cell
+        mem_parameters = self.mem_parameters
+        flag_two_channel = mem_parameters.method == "2channel"
+        if flag_two_channel:
+            l_label_atom = None
+        else:
+            atom_site_susceptibility = crystal.atom_site_susceptibility
+            l_label_atom = atom_site_susceptibility.label
         density_point = self.density_point
         density_point.save_to_file_den(
             mem_parameters, space_group, cell, f_name=f_name,
-            label_atom=label_atom, f_background=f_background)
+            l_label_atom=l_label_atom, f_background=f_background)
 
     def create_prior_density(self):
         """Create prior denisity."""
@@ -115,13 +121,15 @@ class MEM(GlobalN):
             density_point.create_core_density(
                 space_group_symop, cell, atom_site,
                 atom_electron_configuration, points_a=points_a,
-                points_b=points_b, points_c=points_c)
+                points_b=points_b, points_c=points_c,
+                flag_two_channel=flag_two_channel)
         else:
             density_point.create_flat_density(
                 space_group_symop, cell, atom_site,
                 l_magnetic_labes=l_magnetic_labes, points_a=points_a,
                 points_b=points_b, points_c=points_c,
                 flag_two_channel=flag_two_channel)
+
         self.add_items([density_point])
 
     def calc_fr(self):
@@ -140,6 +148,12 @@ class MEM(GlobalN):
         space_group_symop = space_group.full_space_group_symop
         atom_site_susceptibility = crystal.atom_site_susceptibility
 
+        # FIXME: temporary solution of calculation rbs_i if it's not defined
+        density_point.calc_rbs_i(
+            space_group_symop, points_a=mem_parameters.points_a,
+            points_b=mem_parameters.points_b, points_c=mem_parameters.points_c)
+
+
         total_peaks = 0
 
         den_i = numpy.array(density_point.density, dtype=float)
@@ -199,7 +213,6 @@ class MEM(GlobalN):
                                               delta_f_m_perp=f_m_perp)
             diffrn.diffrn_refln.numpy_fr_calc = fr_m
         for diffrn in l_diffrn:
-            # FIXME: not sure that input parameters should be modified
             diffrn.diffrn_refln.numpy_to_items()
             chi_sq, points = diffrn.diffrn_refln.calc_chi_sq_points()
             refine_ls = RefineLs(goodness_of_fit_all=chi_sq/points,
@@ -255,3 +268,29 @@ class MEM(GlobalN):
                        n_iterations=n_iterations, n_cycle=n_cycle, disp=disp,
                        d_info=d_info)
         self.add_items([density_point])
+
+    def calc_moments_in_unit_cell(self, field_loc,
+                                  f_name: str = "moments_in_unit_cell.dat"):
+        """Calc moments in an unit cell.
+
+        If f_name is given, the result is save in file.
+        If it is None, the results are given in output.
+        """
+        density_point = self.density_point
+        crystal = self.crystals()[0]
+        mem_parameters = self.mem_parameters
+        fract_x, fract_y, fract_z, moment_x, moment_y, moment_z = \
+            calc_moments_in_unit_cell(field_loc, density_point, crystal,
+                                      mem_parameters)
+        if f_name is None:
+            return fract_x, fract_y, fract_z, moment_x, moment_y, moment_z
+        else:
+            ls_out = ["# Moments in an unit cell"]
+            ls_out.append("loop_\n_fract_x\n_fract_y\n_fract_z")
+            ls_out.append("_moment_x\n_moment_y\n_moment_z")
+            for f_x, f_y, f_z, m_x, m_y, m_z in zip(
+                    fract_x, fract_y, fract_z, moment_x, moment_y, moment_z):
+                ls_out.append(f"{f_x:9.5f} {f_y:9.5f} {f_z:9.5f} \
+{m_x:10.3f} {m_y:10.3f} {m_z:10.3f}")
+            with open(f_name, "w") as fid:
+                fid.write("\n".join(ls_out))
